//+------------------------------------------------------------------+
//|                                                          Log.mqh |
//|                                     Copyright Paul Hampton-Smith |
//|                            http://paulsfxrandomwalk.blogspot.com |
//+------------------------------------------------------------------+

#include "StringUtilities.mqh"
//+----------------------------------------------------------------------------------+
///        ,  CLog.
//+---------------------------------------------------------------------------------+
enum ENUM_LOG_LEVEL
  {
   /// ,       -  ,  ,      
   LOG_NONE,
   ///    ,          
   LOG_PRINT,
   ///    ,          
   LOG_MAJOR,
   ///      ,            
   LOG_DEBUG,
   ///             
   LOG_VERBOSE
  };
//+------------------------------------------------------------------+
/**       .
 
\b 
	-    Files\\EAlogs\\<EAname>\\<EAname>_<symbol>_yyyymmdd_log.txt
	- 4   , . ENUM_LOG_LEVEL,   LOG_DEBUG
	-      
	-    Print  , ..      
	-           .
	
<b>   </b>

\code
input ENUM_LOG_LEVEL LogLevel = LOG_VERBOSE;

OnInit()
{
	LogFile.LogLevel(LogLevel);
}

bool MyFunction(int param1, double param2)
{
	LOG(LOG_DEBUG,__FUNCTION__,"StringFormat("Entry to MyFunction(%d,%f)",param1,param2)); 
	
	bool bReturn = false;
	
	for (int i=0 ; i<1000 ; i++)
	{
		LOG(LOG_VERBOSE,__FUNCTION__,(string)i);
	}

	if (bReturn)
	{	
 		LOG(LOG_MAJOR,__FUNCTION__,StringFormat("MyFunction(%d,%f) returning true",param1,param2));
	}
	else
	{
		LOG(LOG_PRINT,__FUNCTION__,StringFormat("MyFunction(%d,%f) returning false",param1,param2));
	}
	return(bReturn);
}
\endcode
*/
//+------------------------------------------------------------------+
///    
//+------------------------------------------------------------------+
class CLog
  {
private:
   bool              m_bReducedLogging;            ///< True      
   ENUM_LOG_LEVEL    m_LogLevel;                   ///<  ,   LogLevel().
   string            m_strFilename;                ///<    ,  Log(),   Filename().
   uint              m_uLogFileMaxAgeDays;         ///<    ,         .   MaxLogAgeDays()
   uint              m_uLogStartDaysSince1970;     ///<     .
   ulong             m_ulMaxLogfileSize;           ///<        ,      .   MaxLogFileSizeMB()

protected:
   void              CleanUpOldLogs();             ///<    ,  ,  MaxLogAgeDays().
   string            MakeLogFilename(datetime dt); ///<     .
   string            MakeLogFilenameBase();        ///<    .
   datetime          LogFileDate(string strLogFilename); ///<      .
   string            LogLevelString();             ///<  ENUM_LOG_LEVEL  .
   string            PeriodString();               ///<     ( M1, H1, D1  .).

public:
                     CLog(); ///<     .
   void              Delete(){ if(FileIsExist(m_strFilename)) FileDelete(m_strFilename); } ///<    .
   string            Filename(){return(m_strFilename);} ///<    ,  Log().
                     ///    .
   void              Log(const ENUM_LOG_LEVEL Level,string s1="",string s2="",string s3="",string s4="",string s5="",string s6="",string s7="",string s8="",string s9="",string s10="");
   ///   .
   ENUM_LOG_LEVEL    LogLevel(const ENUM_LOG_LEVEL Level)
     {
      m_LogLevel=Level;
      Print("Logging at level",LogLevelString(),"to file",Filename());
      return(m_LogLevel);
     }
   ENUM_LOG_LEVEL    LogLevel() {return(m_LogLevel);} ///<   .
   uint              MaxLogFileSizeMB(uint uMaxSize){ m_ulMaxLogfileSize=uMaxSize*1024*1024; return(uMaxSize);} ///<      MB
   uint              MaxLogFileSizeMB(){ return((uint)(m_ulMaxLogfileSize/(1024*1024)));} ///<      MB
                                                                                          ///     ,        .
   uint              MaxLogAgeDays(const uint uDays){ return(m_uLogFileMaxAgeDays=uDays);}
   ///   ,          
   uint              MaxLogAgeDays(){ return(m_uLogFileMaxAgeDays);}
  };
//+------------------------------------------------------------------+
///     .
///         28  - MaxLogAgeDays() \n
///         50MB - MaxLogFileSizeMB() \n
///     LogLevel  LOG_DEBUG - LogLevel() \n
//+------------------------------------------------------------------+
CLog::CLog()
  {
   m_strFilename=MakeLogFilename(TimeLocal());
   MaxLogAgeDays(28);
   MaxLogFileSizeMB(50);
   m_LogLevel=LOG_DEBUG;
   m_bReducedLogging=false;
   m_uLogStartDaysSince1970=0;
  }
//+------------------------------------------------------------------+
///    .
///    Files\\EAlogs\\<EAname>\\<EAname>_<symbol>_<period>_yyyymmdd_log.txt \n
///         \n
///    ,  MaxLogAgeDays() \n
///     ,     MaxLogFileSizeMB()
/// \param [in]   Level    				  
/// \param [in]   s1,s2...,s10     10   
//+------------------------------------------------------------------+
void CLog::Log(const ENUM_LOG_LEVEL Level,string s1="",string s2="",string s3="",string s4="",string s5="",string s6="",string s7="",string s8="",string s9="",string s10="")
  {
   if(Level>m_LogLevel)return;
   //      
   if((bool)MQL5InfoInteger(MQL5_OPTIMIZATION)) return;

   string strMsg; StringInit(strMsg);
   StringConcatenate(strMsg,s1,s2,s3,s4,s5,s6,s7,s8,s9,s10);

   //        Print
   if((bool)MQL5InfoInteger(MQL5_TESTING))
     {
      Print(strMsg);
      return;
     }

   datetime dtNow=TimeLocal();
   uint uDaysSince1970=(uint)dtNow/(3600*24);
   if(uDaysSince1970>m_uLogStartDaysSince1970)
     {
      //  ,  
      m_strFilename=MakeLogFilename(dtNow);
      m_uLogStartDaysSince1970=uDaysSince1970;
      CleanUpOldLogs();
     }

   int hLog=FileOpen(m_strFilename,FILE_TXT|FILE_READ|FILE_WRITE);
   if(hLog<=0)
     {
      Print("Warning - logfile problem.",strMsg);
     }
   else
     {
      //      
      FileSeek(hLog,0,SEEK_END);
      FileWrite(hLog,TimeToString(dtNow,TIME_DATE|TIME_SECONDS)+" "+strMsg);

      //        LOG_PRINT
      if(Level==LOG_PRINT) Print(strMsg);

      //    
      if(!m_bReducedLogging && FileSize(hLog)>m_ulMaxLogfileSize)
        {
         //       ,     
         m_bReducedLogging=true;
         if(LogLevel()==LOG_MAJOR || LogLevel()==LOG_PRINT)
           {
            Print("Warning - max log size of",MaxLogFileSizeMB(),"MB has been reached, no further logging active");
            LogLevel(LOG_NONE);
           }
         else
           {
            Print("Warning - max log size of",MaxLogFileSizeMB(),"MB has been reached, reducing log level");
            LogLevel(LOG_MAJOR);
           }
        }
      FileClose(hLog);
     }
  }
//+------------------------------------------------------------------+
///    .
///      (wildcard)      
/// \return				The filename base EAlogs\\<EAname>\\<EAname>_<symbol>_<period>
//+------------------------------------------------------------------+
string CLog::MakeLogFilenameBase()
  {
   string strName=MQL5InfoString(MQL5_PROGRAM_NAME);
   return(StringFormat("EAlogs\\%s\\%s_%s_%s",strName,strName,StringSubstr(_Symbol,0,6),PeriodString()));
  }
//+------------------------------------------------------------------+
///     .
/// \param [in] dt	,       
/// \return				  EAlogs\\<EAname>\\<EAname>_<symbol>_yyyymmdd_log.txt
//+------------------------------------------------------------------+
string CLog::MakeLogFilename(datetime dt)
  {
   MqlDateTime mdt;
   TimeToStruct(dt,mdt);
   return(StringFormat("%s_%d%02d%02d_log.txt",MakeLogFilenameBase(),mdt.year,mdt.mon,mdt.day));
  }
//+------------------------------------------------------------------+
///    ,     MaxLogAgeDays().
//+------------------------------------------------------------------+
void CLog::CleanUpOldLogs()
  {
   datetime dtNow=TimeLocal();
   datetime dtExpiry=dtNow-24*3600*MaxLogAgeDays();
   string strFilenameWildcard=MakeLogFilenameBase()+"*";
   string strFoundFile="";

   long hFind=FileFindFirst(strFilenameWildcard,strFoundFile);
   if(hFind!=INVALID_HANDLE)
     {
      do
        {
         strFoundFile="EAlogs\\"+MQL5InfoString(MQL5_PROGRAM_NAME)+"\\"+strFoundFile;
         datetime dt=LogFileDate(strFoundFile);
         if(dt!=-1 && dt<dtExpiry)
           {
            ResetLastError();
            FileDelete(strFoundFile);
            Print(ErrorDescription(GetLastError()),"cleaning up >",MaxLogAgeDays(),"day old logfile",strFoundFile);
           }
        }
      while(FileFindNext(hFind,strFoundFile));
      FileFindClose(hFind);
     }
  }
//+------------------------------------------------------------------
///      LogFilename.
/// \param [in] strLogFilename    
/// \return                        
//+------------------------------------------------------------------+
datetime CLog::LogFileDate(string strLogFilename)
  {
   int len=StringLen(strLogFilename);
   MqlDateTime mdt;
// ...yyyymmdd_log.txt
   mdt.year = (int)StringToInteger(StringSubstr(strLogFilename,len-16,4));
   mdt.mon  = (int)StringToInteger(StringSubstr(strLogFilename,len-12,2));
   mdt.day  = (int)StringToInteger(StringSubstr(strLogFilename,len-10,2));
   mdt.hour = 0;
   mdt.min  = 0;
   mdt.sec  = 0;
   return(StructToTime(mdt));
  }
//+------------------------------------------------------------------+
/// Converts ENUM_LOG_LEVEL into string.
//+------------------------------------------------------------------+
string CLog::LogLevelString()
  {
   switch(m_LogLevel)
     {
      case LOG_NONE: return("LOG_NONE");
      case LOG_PRINT: return("LOG_PRINT");
      case LOG_MAJOR: return("LOG_MAJOR");
      case LOG_DEBUG: return("LOG_DEBUG");
      case LOG_VERBOSE: return("LOG_VERBOSE");
      default: return("error: Unknown log level "+(string)LogLevel());
     }
  }
//+------------------------------------------------------------------+
///   (M1, H1, D1  ..)
//+------------------------------------------------------------------+
string CLog::PeriodString()
  {
   switch(_Period)
     {
      case PERIOD_M1: return("M1");
      case PERIOD_M2: return("M2");
      case PERIOD_M3: return("M3");
      case PERIOD_M4: return("M4");
      case PERIOD_M5: return("M5");
      case PERIOD_M6: return("M6");
      case PERIOD_M10: return("M10");
      case PERIOD_M12: return("M12");
      case PERIOD_M15: return("M15");
      case PERIOD_M20: return("M20");
      case PERIOD_M30: return("M30");
      case PERIOD_H1: return("H1");
      case PERIOD_H2: return("H2");
      case PERIOD_H3: return("H3");
      case PERIOD_H4: return("H4");
      case PERIOD_H6: return("H6");
      case PERIOD_H8: return("H8");
      case PERIOD_D1: return("D1");
      case PERIOD_W1: return("W1");
      case PERIOD_MN1: return("MN1");
      default: return("error: Unknown period "+(string)_Period);
     }
  }
// Define global logfile
CLog LogFile;
#define LOG LogFile.Log
//+------------------------------------------------------------------+
