//+------------------------------------------------------------------+
//|                                          VirtualOrderManager.mqh |
//|                                     Copyright Paul Hampton-Smith |
//|                            http://paulsfxrandomwalk.blogspot.com |
//+------------------------------------------------------------------+
#property copyright "Paul Hampton-Smith"
#property link      "http://paulsfxrandomwalk.blogspot.com"

#include "Log.mqh"
#include "VirtualOrderManagerEnums.mqh"
#include "StringUtilities.mqh"
#include "VirtualOrderArray.mqh"
#include "VirtualOrder.mqh"
#include "VirtualOrderManagerConfig.mqh"
#include "SimpleChartObject.mqh"
#include "GlobalVirtualStopList.mqh"
#include "VOM_manual.mqh"
//+------------------------------------------------------------------+
///       .
//+------------------------------------------------------------------+
class CVirtualOrderManager
  {
private:
   CVirtualOrder    *m_SelectedOrder; ///<   OrderSelect()
   long              m_lMagicNumber;  ///< Magic number ,   VOM
   int               m_nLastBars;     ///< -    ,   OnTick()
   bool              m_bNewBar;       ///<   true  OnTick()      , NewBar()
   datetime          m_dtTime0;       ///<     
   CGlobalVirtualStopList m_GlobalVirtualStopList;

public:
   CVirtualOrderArray m_OpenOrders;    ///<       VOM,   
   CVirtualOrderArray m_OrderHistory;  ///<       VOM,   

                                      /// ,   .
                    ~CVirtualOrderManager();

	///      
   void BroadcastVirtualTradeEvent();
   ///    buy.
   long              Buy(string strSymbol,double dblLots,const int nStoploss,const int nTakeprofit,string strComment="");
   ///       
   bool              CloseAllOrders(string strSymbol,ENUM_VIRTUAL_ORDER_TYPE Type);
   ///    OnInit() .
   void              Initialise(long lMagicNumber=-1);
   ///       ?
   bool              LastOrderCloseInBar(string strSymbol="");
   /// Magic number  ,   VOM
   long             MagicNumber(){return(m_lMagicNumber);}
   ///   magic number,      .
   long              MakeMagic(string strSymbol="");
   /// True     .
   bool NewBar(){return(m_bNewBar);}
   ///    OnTick() .
   void              OnTick();
   ///  (-   * 1000) 
   int               OpenLots(string strSymbol="");
   ///   
   int               OpenOrders(string strSymbol=""){if(strSymbol=="")strSymbol=_Symbol;return(m_OpenOrders.OrderCount(strSymbol,MagicNumber()));}
   /// ,     .
   int               OpenedOrdersInSameBar(string strSymbol="");
   ///   .
   bool              OrderClose(long ticket,int slippage,color Color=CLR_NONE);
   ///     
   bool OrderCloseBy(long ticket,int opposite,color Color=CLR_NONE) {LogFile.Log(LOG_PRINT,__FUNCTION__,"Not implemented"); return(false);}
   ///    .    OrderSelect()
   double OrderClosePrice() {if(ValidSelectedOrder()) return(m_SelectedOrder.ClosePrice()); else return(0.0);}
   ///    .    OrderSelect()
   datetime OrderCloseTime() {if(ValidSelectedOrder()) return(m_SelectedOrder.CloseTime()); else return((datetime)0);}
   ///   .    OrderSelect()
   string OrderComment() {if(ValidSelectedOrder()) return(m_SelectedOrder.Comment()); else return("");}
   double OrderCommission() {if(ValidSelectedOrder()) return(m_SelectedOrder.Commission()); else return(0.0);}
   ///    .
   bool              OrderDelete(long ticket);
   ///      .    OrderSelect()
   datetime OrderExpiration() {if(ValidSelectedOrder()) return(m_SelectedOrder.Expiration()); else return((datetime)0);}
   ///   .    OrderSelect()
   double OrderLots() {if(ValidSelectedOrder()) return(m_SelectedOrder.Lots()); else return(0.0);}
   /// Magic number  .    OrderSelect()
   long OrderMagicNumber() {if(ValidSelectedOrder()) return(m_SelectedOrder.MagicNumber()); else return(0);}
   ///   .
   bool              OrderModify(long ticket,double price,int nStopLoss,int nTakeProfit,datetime expiration,color arrow_color=CLR_NONE);
   bool              OrderModify(long ticket,double price,double stoploss,double takeprofit,datetime expiration,color arrow_color=CLR_NONE);
   ///     .    OrderSelect()
   double OrderOpenPrice() {if(ValidSelectedOrder()) return(m_SelectedOrder.OpenPrice()); else return(0.0);}
   ///     .    OrderSelect()
   datetime OrderOpenTime() {if(ValidSelectedOrder()) return(m_SelectedOrder.OpenTime()); else return((datetime)0);}
   ///    .    OrderSelect()
   double OrderProfit() {if(ValidSelectedOrder()) return(m_SelectedOrder.Profit()); else return(0.0);}
   ///         
   bool              OrderSelect(long index,ENUM_VIRTUAL_SELECT_TYPE type,ENUM_VIRTUAL_SELECT_MODE pool=MODE_TRADES);
   ///      .
   long              OrderSend(string symbol,ENUM_VIRTUAL_ORDER_TYPE cmd,double volume,double price,int slippage,double stoploss,double takeprofit,string comment="",datetime expiration=0,color arrow_color=CLR_NONE);
   ///      
   int OrdersHistoryTotal() {return(m_OrderHistory.Total());}
   ///  Stop Loss   .    OrderSelect()
   double OrderStopLoss() {if(ValidSelectedOrder()) return(m_SelectedOrder.StopLoss()); else return(0.0);}
   ///    (Symbol)   .    OrderSelect()
   string OrderSymbol() {if(ValidSelectedOrder()) return(m_SelectedOrder.Symbol()); else return("");}
   ///  Take Profit   .    OrderSelect()
   double OrderTakeProfit() {if(ValidSelectedOrder()) return(m_SelectedOrder.TakeProfit()); else return(0.0);}
   ///    .
   int OrdersTotal() {return(m_OpenOrders.Total());}
   /// Ticket for selected virtual order. Use OrderSelect() first
   ///    .    OrderSelect()
   long OrderTicket() {return(m_SelectedOrder.Ticket());}
   ///    .    OrderSelect()
   int OrderType() {return(m_SelectedOrder.OrderType());}
   /// Sends Buy or Sell orders to adjust real server position size.
   ///      .
   bool              PositionChangeSizeAtServer(string strSymbol,double dblLots,ENUM_ORDER_TYPE OrderType,double dblStopLoss);
   ///    StopLoss.
   bool              PositionSetTightestStopAtServer(string strSymbol);
   ///    Sell
   long              Sell(string strSymbol,double dblLots,const int nStoploss,const int nTakeprofit,string strComment="");
   ///     
   datetime            Time0(){return(m_dtTime0);}
   ///    
   bool ValidSelectedOrder()
     {
      if(CheckPointer(m_SelectedOrder)==POINTER_INVALID)
        {
         LogFile.Log(LOG_PRINT,__FUNCTION__,"Error: m_SelectedOrder pointer is not valid");
         return(false);
        }
      else
        {
         return(true);
        }
     }
  };

//    
CVirtualOrderManager VOM;
//+------------------------------------------------------------------+
/// ,   .
//+------------------------------------------------------------------+
CVirtualOrderManager::~CVirtualOrderManager()
  {
   if(CheckPointer(m_SelectedOrder)!=POINTER_INVALID) delete(m_SelectedOrder);
  }
//+-----------------------------------------------------------------------------+
///          .
//+-----------------------------------------------------------------------------+
void CVirtualOrderManager::BroadcastVirtualTradeEvent()
  {
   long lChartID=ChartFirst();
   while(lChartID>=0)
     {
      if(lChartID!=ChartID())
        {
         LogFile.Log(LOG_DEBUG,__FUNCTION__);
         EventChartCustom(lChartID,Config.CustomVirtualOrderEvent,0,0.0,"");
        }
      lChartID=ChartNext(lChartID);
     }
  }
//+------------------------------------------------------------------+
///    Buy.
/// \param strSymbol		 
/// \param nStopLoss		 Stoploss     
/// \param nTakeProfit	 Takeprofit     
/// \param strComment	   
/// \return 				     ,  -1   .
//+------------------------------------------------------------------+
long CVirtualOrderManager::Buy(string strSymbol,double dblLots,const int nStoploss,const int nTakeprofit,string strComment/*=""*/)
  {
   if(strComment=="") strComment=MQL5InfoString(MQL5_PROGRAM_NAME);
   LogFile.Log(LOG_DEBUG,__FUNCTION__,StringFormat("(%s,%0.2f,%d,%d,%s)",strSymbol,dblLots,nStoploss,nTakeprofit,strComment));
   double sl,tp;
   double dblPrice=SymbolInfoDouble(strSymbol,SYMBOL_ASK);
   double dblPoint=SymbolInfoDouble(strSymbol,SYMBOL_POINT);
   sl =nStoploss==0 ? 0.0 : dblPrice-nStoploss*dblPoint;
   tp =nTakeprofit==0 ? 0.0 : dblPrice+nTakeprofit*dblPoint;
   return(VOM.OrderSend(strSymbol,VIRTUAL_ORDER_TYPE_BUY,dblLots,dblPrice,Config.Deviation,sl,tp,strComment,0));
  }
//+------------------------------------------------------------------+
///       
/// \param [in] strSymbol  
/// \param [in] Type			  
/// \return						     True  ,    false.
//+------------------------------------------------------------------+
bool CVirtualOrderManager::CloseAllOrders(string strSymbol,ENUM_VIRTUAL_ORDER_TYPE Type)
  {
   for(int i=m_OpenOrders.Total()-1;i>=0;i--)
     {
      CVirtualOrder *vo=m_OpenOrders.VirtualOrder(i);
      if(vo.MagicNumber()==MagicNumber())
         if(vo.OrderType()==Type)
            if(vo.Symbol()==strSymbol)
               if(!OrderClose(vo.Ticket(),Config.Deviation))
                 {
                  return(false);
                 }
     }
   return(true);
  }
//+------------------------------------------------------------------+
///    OnInit() .
///      \n
///      ,  VirtualOrderManager
/// \code
/// // EA code
/// void OnInit()
///  {
///   VOM.Initialise();
///   //
///   //    
///   // ....
/// \endcode
//+------------------------------------------------------------------+
void CVirtualOrderManager::Initialise(long lMagicNumber/*=-1*/)
  {
// restore previous state
   m_OpenOrders.PersistFilename(StringFormat("%s%s_%s_%s_OpenOrders.csv",Config.FileBase,_Symbol,PeriodToString(_Period),MQL5InfoString(MQL5_PROGRAM_NAME)));
   m_OpenOrders.Clear();
   m_OpenOrders.ReadFromFile();

   m_OrderHistory.PersistFilename(StringFormat("%s%s_%s_%s_OrderHistory.csv",Config.FileBase,_Symbol,PeriodToString(_Period),MQL5InfoString(MQL5_PROGRAM_NAME)));
   m_OrderHistory.Clear();
   m_OrderHistory.ReadFromFile();

   m_nLastBars=Bars(_Symbol,Period());
   m_bNewBar=false;
   if(lMagicNumber==-1)
      MakeMagic();
   else
      m_lMagicNumber=lMagicNumber;
   LogFile.Log(LOG_PRINT,__FUNCTION__," - System ID or Magic Number is ",(string)m_lMagicNumber);

   datetime Arr[];
   if(CopyTime(_Symbol,_Period,0,1,Arr)!=1)
     {
      m_dtTime0=0;
      LogFile.Log(LOG_MAJOR,__FUNCTION__," error - unable to obtain bar time");
     }
   else
     {
      m_dtTime0=Arr[0];
     }
  }
//+------------------------------------------------------------------+
///       ?
//+------------------------------------------------------------------+
bool CVirtualOrderManager::LastOrderCloseInBar(string strSymbol/*=""*/)
  {
   if(strSymbol=="") strSymbol=_Symbol;
//     100      
   int nMaxLookback=(int)MathMax(0,OrdersHistoryTotal()-100);

   for(int nPosition=OrdersHistoryTotal()-1;nPosition>=nMaxLookback;nPosition--)
     {
      CVirtualOrder *vo=m_OrderHistory.VirtualOrder(nPosition);
      if(vo.MagicNumber()==MagicNumber())
         if(vo.Symbol()==strSymbol)
           {
            return(vo.CloseTime()>=Time0());
           }
     }
   return(false);
  }
//+------------------------------------------------------------------+
///   magic number,      .
///  - , : 
/// - http://forum.mql4.com/25432/page4
/// - http://www.cse.yorku.ca/~oz/hash.html
//+------------------------------------------------------------------+
long CVirtualOrderManager::MakeMagic(string strSymbol/*=""*/)
  {
   if(strSymbol=="") strSymbol=_Symbol;
   string s=strSymbol+PeriodToString(Period())+MQL5InfoString(MQL5_PROGRAM_NAME);
   ulong ulHash=5381;
   for(int i=StringLen(s)-1;i>=0;i--)
     {
      ulHash=((ulHash<<5)+ulHash)+StringGetCharacter(s,i);
     }
//     long          MT5 functions   ulong
   m_lMagicNumber=(long)ulHash;
   if(m_lMagicNumber<0) m_lMagicNumber=-m_lMagicNumber;
   return(m_lMagicNumber);
  }
//+------------------------------------------------------------------+
///    OnTick() ..
///   Stop Loss  Take Profit \n
///   ,  VirtualOrderManager   :
/// \code
/// //  
/// void OnTick()
///  {
///   //   stoploss, takeprofit
///   VOM.OnTick();
///   //
///   //           
///   // ....
/// \endcode
//+------------------------------------------------------------------+
void CVirtualOrderManager::OnTick()
  {
   uint uDaysSince1970=(uint)TimeLocal()/(3600*24);
   static uint uLastDaysSince1970=0;
   if(uLastDaysSince1970<uDaysSince1970)
     {
      LogFile.Log(LOG_PRINT,__FUNCTION__," - startup tick or first local time tick of new day");
      uLastDaysSince1970=uDaysSince1970;
     }

//  m_bNewBar  true       
   int nBars=Bars(_Symbol,PERIOD_CURRENT);
   if(m_nLastBars!=nBars)
     {
      m_nLastBars=nBars;
      m_bNewBar=true;
      datetime Arr[];
      if(CopyTime(_Symbol,_Period,0,1,Arr)!=1)
        {
         m_dtTime0=0;
         LogFile.Log(LOG_MAJOR,__FUNCTION__," error - unable to obtain bar time");
        }
      else
        {
         m_dtTime0=Arr[0];
        }
     }
   else
     {
      m_bNewBar=false;
     }

//   sl,tp,or    
   for(int i=m_OpenOrders.Total()-1;i>=0;i--)
     {
      CVirtualOrder *vo=m_OpenOrders.VirtualOrder(i);
      if(vo.MagicNumber()==MagicNumber())
         if(vo.Symbol()==_Symbol)
           {
            switch(vo.CheckStopsAndLimits())
              {
               case VIRTUAL_ORDER_EVENT_NONE:
                  break;

               case VIRTUAL_ORDER_EVENT_BUY_TRIGGER:
                  if(PositionChangeSizeAtServer(vo.Symbol(),vo.Lots(),ORDER_TYPE_BUY,vo.StopLoss()))
                    {
                     vo.PendingOrderTrigger();
                     m_OpenOrders.WriteToFile();
                     BroadcastVirtualTradeEvent();
                     PositionSetTightestStopAtServer(_Symbol);
                    }
                  break;

               case VIRTUAL_ORDER_EVENT_SELL_TRIGGER:
                  if(PositionChangeSizeAtServer(vo.Symbol(),vo.Lots(),ORDER_TYPE_SELL,vo.StopLoss()))
                    {
                     vo.PendingOrderTrigger();
                     m_OpenOrders.WriteToFile();
                     BroadcastVirtualTradeEvent();
                     PositionSetTightestStopAtServer(_Symbol);
                    }
                  break;

               case VIRTUAL_ORDER_EVENT_STOPLOSS:
               case VIRTUAL_ORDER_EVENT_TAKEPROFIT:
                  OrderClose(vo.Ticket(),Config.Deviation);
                  break;

               default:
                  LogFile.Log(LOG_PRINT,__FUNCTION__," error: invalid Virtual Order Event");
              } // switch vo.CheckStopsAndLimits()
           } // if
     } // i
  }
//+------------------------------------------------------------------+
/// ,     .
//+------------------------------------------------------------------+
int CVirtualOrderManager::OpenedOrdersInSameBar(string strSymbol/*=""*/)
  {
   if(strSymbol=="") strSymbol=_Symbol;
   int nOpenOrders=0;

   for(int nPosition=OrdersTotal()-1;nPosition>=0;nPosition--)
     {
      CVirtualOrder *vo=m_OpenOrders.VirtualOrder(nPosition);
      if(vo.MagicNumber()==MagicNumber())
         if(vo.Symbol()==strSymbol)
           {
            if(vo.OpenTime()>=Time0()) nOpenOrders++;
           }
     }
   return(nOpenOrders);
  }
//+------------------------------------------------------------------+
///   .
/// \param [in] ticket			   
/// \param [in] slippage		,    deviation.  50
/// \param [in] arrow_color   CLR_NONE.    ,     MT4.
/// \return							    true  , false  
//+------------------------------------------------------------------+
bool CVirtualOrderManager::OrderClose(long ticket,int slippage,color Color=CLR_NONE)
  {
   LogFile.Log(LOG_DEBUG,__FUNCTION__,"(",(string)ticket,",",(string)slippage,")");
   int nVoIndex=m_OpenOrders.TicketToIndex(ticket);
   if(nVoIndex==-1)
     {
      LogFile.Log(LOG_DEBUG,__FUNCTION__," warning: invalid ticket - no action");
      return(false);
     }
   CVirtualOrder *vo=m_OpenOrders.VirtualOrder(nVoIndex);
   m_GlobalVirtualStopList.Delete(vo.Ticket());

   bool bSuccess=false;
   switch(vo.OrderType())
     {
      case VIRTUAL_ORDER_TYPE_BUY:
         bSuccess=PositionChangeSizeAtServer(vo.Symbol(),vo.Lots(),ORDER_TYPE_SELL,0);
         break;
      case VIRTUAL_ORDER_TYPE_SELL:
         bSuccess=PositionChangeSizeAtServer(vo.Symbol(),vo.Lots(),ORDER_TYPE_BUY,0);
         break;
      default:
         LogFile.Log(LOG_PRINT,__FUNCTION__," error: invalid ordertype");
     }

   if(bSuccess)
     {
      //             
      vo.Close();
      //   ,    
      m_OpenOrders.Detach(nVoIndex);
      m_OrderHistory.Add(vo);
      PositionSetTightestStopAtServer(vo.Symbol());
      m_OpenOrders.WriteToFile();
      m_OrderHistory.WriteToFile();
      BroadcastVirtualTradeEvent();
      return(true);
     }
   else
     {
      //  stoploss ( )
      m_GlobalVirtualStopList.Add(vo.Ticket(),vo.Symbol(),vo.OrderType(),vo.StopLoss());
      return(false);
     }
  }
//+------------------------------------------------------------------+
///    .
//+------------------------------------------------------------------+
bool CVirtualOrderManager::OrderDelete(long ticket)
  {
   LogFile.Log(LOG_DEBUG,__FUNCTION__,"(",(string)ticket,")");
   int nVoIndex=m_OpenOrders.TicketToIndex(ticket);

   if(nVoIndex==-1)
     {
      LogFile.Log(LOG_DEBUG,__FUNCTION__," warning: invalid ticket - no action");
      return(false);
     }

   CVirtualOrder *vo=m_OpenOrders.VirtualOrder(nVoIndex);
   if(vo.OrderType()==VIRTUAL_ORDER_TYPE_BUY || vo.OrderType()==VIRTUAL_ORDER_TYPE_SELL || vo.Status()==VIRTUAL_ORDER_STATUS_OPEN)
     {
      LogFile.Log(LOG_PRINT,__FUNCTION__," error: cannot delete an open order");
      return(false);
     }

//           
   vo.Delete();
   //   ,    
   m_OpenOrders.Detach(nVoIndex);
   m_OpenOrders.WriteToFile();
      BroadcastVirtualTradeEvent();
   
   m_OrderHistory.Add(vo);
   m_OrderHistory.WriteToFile();
   return(true);
  }
//+------------------------------------------------------------------+
///   .
/// <b>       .</b>
/// \param [in] ticket			  
/// \param [in] price			   
/// \param [in] nStopLoss		  Stop Loss       0,  
/// \param [in] nTakeProfit	  Take Profit       0,  
/// \param [in] expiration 	  0 (1/1/1970)
/// \param [in] arrow_color   CLR_NONE.    ,     MT4.
/// \return							    true  , false  
//+------------------------------------------------------------------+
bool CVirtualOrderManager::OrderModify(long ticket,double price,int nStopLoss,int nTakeProfit,datetime expiration,color arrow_color=CLR_NONE)
  {
   LogFile.Log(LOG_DEBUG,__FUNCTION__,StringFormat("(%d,%0.5f,%d,%d,%s,%s)",ticket,price,nStopLoss,nTakeProfit,TimeToString(expiration)));

   int nIndex=m_OpenOrders.TicketToIndex(ticket);
   if(nIndex<0) return(false);

   CVirtualOrder *vo=m_OpenOrders.VirtualOrder(nIndex);
   double stoploss=0,takeprofit=0;
   double dblPoint=SymbolInfoDouble(vo.Symbol(),SYMBOL_POINT);

   if(price==0.0) price=vo.OpenPrice();

   switch(vo.OrderType())
     {
      case VIRTUAL_ORDER_TYPE_BUYSTOP:
      case VIRTUAL_ORDER_TYPE_BUYLIMIT:
      case VIRTUAL_ORDER_TYPE_BUY:
         stoploss=nStopLoss==0 ? 0.0 : price-nStopLoss*dblPoint;
         takeprofit=nTakeProfit==0 ? 0.0 : price+nTakeProfit*dblPoint;
         break;

      case VIRTUAL_ORDER_TYPE_SELLSTOP:
      case VIRTUAL_ORDER_TYPE_SELLLIMIT:
      case VIRTUAL_ORDER_TYPE_SELL:
         stoploss=nStopLoss==0 ? 0.0 : price+nStopLoss*dblPoint;
         takeprofit=nTakeProfit==0 ? 0.0 : price-nTakeProfit*dblPoint;
         break;

      default:
         LogFile.Log(LOG_PRINT,__FUNCTION__," error: invalid order type");
     }

   return(OrderModify(ticket,price,stoploss,takeprofit,expiration,arrow_color));
  }
//+------------------------------------------------------------------+
///   .
/// <b>       .</b>
/// \param [in] ticket			   
/// \param [in] price			   
/// \param [in] stoploss		 Stop Loss,  ,  0
/// \param [in] takeprofit	 Take Profit,  ,  0
/// \param [in] expiration 	  0 (1/1/1970)
/// \param [in] arrow_color   CLR_NONE.     ,     MT4.
/// \return							    true  , false  
//+------------------------------------------------------------------+
bool CVirtualOrderManager::OrderModify(long ticket,double price,double stoploss,double takeprofit,datetime expiration,color arrow_color=CLR_NONE)
  {
   LogFile.Log(LOG_DEBUG,__FUNCTION__,StringFormat("(%d,%0.5f,%0.5f,%0.5f,%s,%s)",ticket,price,stoploss,takeprofit,TimeToString(expiration)));
   CVirtualOrder *vo=m_OpenOrders.AtTicket(ticket);
   if(vo==NULL)
     {
      LogFile.Log(LOG_PRINT,__FUNCTION__," error: virtual order ticket ",(string)ticket," not found");
      return(false);
     }

   bool bSuccess=vo.Modify(price,stoploss,takeprofit,expiration);

   if(bSuccess)
     {
      if(vo.Status()==VIRTUAL_ORDER_STATUS_OPEN)
        {
         if(!PositionSetTightestStopAtServer(vo.Symbol()))
           {
            LogFile.Log(LOG_PRINT,__FUNCTION__," error: failed to modify stoploss at server");
            return(false);
           }
        }
      m_OpenOrders.WriteToFile();
      BroadcastVirtualTradeEvent();
      LogFile.Log(LOG_DEBUG,__FUNCTION__," success");
      return(true);
     }
   else
     {
      LogFile.Log(LOG_DEBUG,__FUNCTION__," failure");
      return(false);
     }
  }
//+------------------------------------------------------------------+
///    
/// \param [in] i			 ,    
/// \param [in] type	 SELECT_BY_POS  SELECT_BY_TICKET		
/// \param [in] pool	MODE_TRADES ( )  MODE_HISTORY
/// \return					  true  ,  false
//+------------------------------------------------------------------+
bool CVirtualOrderManager::OrderSelect(long i,ENUM_VIRTUAL_SELECT_TYPE type,ENUM_VIRTUAL_SELECT_MODE pool=MODE_TRADES)
  {
   switch(type)
     {
      case SELECT_BY_POS:
         switch(pool)
           {
            case MODE_TRADES: m_SelectedOrder=m_OpenOrders.VirtualOrder(int(i)); return(true);
            case MODE_HISTORY: m_SelectedOrder=m_OrderHistory.VirtualOrder(int(i)); return(true);
            default:
               LogFile.Log(LOG_PRINT,__FUNCTION__," error: Unknown pool id ",(string)pool);
               return(false);
           }
         break;
      case SELECT_BY_TICKET:
         switch(pool)
           {
            case MODE_TRADES: m_SelectedOrder=m_OpenOrders.AtTicket(i); return(true);
            case MODE_HISTORY: m_SelectedOrder=m_OrderHistory.AtTicket(i); return(true);
            default:
               LogFile.Log(LOG_PRINT,__FUNCTION__," error: Unknown pool id ",(string)pool);
               return(false);
           }
         break;
      default:
         LogFile.Log(LOG_PRINT,__FUNCTION__," error: Unknown type ",(string)type);
         return(false);
     }
  }
//+------------------------------------------------------------------+
///      .
/// \param [in] symbol		 .    (. EURUSD)    (. EURUSDm) 
/// \param [in] cmd				VIRTUAL_ORDER_TYPE_BUY or VIRTUAL_ORDER_TYPE_SELL 
/// \param [in] volume		 
/// \param [in] price			 
/// \param [in] slippage		,  ,  deviation.    50
/// \param [in] stoploss		 Stop loss,  0   
/// \param [in] takeprofit	 Take Profit,  0   
/// \param [in] comment			   ,   
/// \param [in] expiration 	  0 (1/1/1970)
/// \param [in] arrow_color   CLR_NONE.    ,      MT4.
/// \return							Order ticket if successful, -1 if not
//+------------------------------------------------------------------+
long CVirtualOrderManager::OrderSend(string symbol,ENUM_VIRTUAL_ORDER_TYPE cmd,double volume,double price,int slippage,double stoploss,double takeprofit,string comment="",datetime expiration=0,color arrow_color=CLR_NONE)
  {
   LogFile.Log(LOG_DEBUG,__FUNCTION__,StringFormat("(%s,%s,%0.2f,%0.5f,%d,%0.5f,%0.5f,%s,%s)",symbol,VirtualOrderTypeToStr(cmd),volume,price,slippage,stoploss,takeprofit,comment,TimeToString(expiration)));

   bool bSuccess=false;
   switch(cmd)
     {
      case VIRTUAL_ORDER_TYPE_BUY:
         bSuccess=PositionChangeSizeAtServer(symbol,volume,ORDER_TYPE_BUY,stoploss); break;
      case VIRTUAL_ORDER_TYPE_SELL:
         bSuccess=PositionChangeSizeAtServer(symbol,volume,ORDER_TYPE_SELL,stoploss); break;

      case VIRTUAL_ORDER_TYPE_BUYSTOP:
      case VIRTUAL_ORDER_TYPE_BUYLIMIT:
      case VIRTUAL_ORDER_TYPE_SELLSTOP:
      case VIRTUAL_ORDER_TYPE_SELLLIMIT:
         // no action at server
         bSuccess=true;
         break;

      default:
         LogFile.Log(LOG_PRINT,__FUNCTION__," error: Invalid ENUM_VIRTUAL_ORDER_TYPE");
         return(-1);
     }

   if(bSuccess)
     {
      CVirtualOrder *vo=new CVirtualOrder;
      vo.Send(symbol,cmd,volume,price,stoploss,takeprofit,comment,MagicNumber(),expiration);
      m_OpenOrders.Add(vo);
      m_OpenOrders.WriteToFile();
            BroadcastVirtualTradeEvent();

      LogFile.Log(LOG_PRINT,__FUNCTION__,"(",vo.Trace(),") returning ",(string)vo.Ticket());
      PositionSetTightestStopAtServer(symbol);
      return(vo.Ticket());
     }
   else
     {
      LogFile.Log(LOG_PRINT,__FUNCTION__," error: failed to adjust position at server, returning -1");
      return(-1);
     }
  }
//+------------------------------------------------------------------------------------------+
///   - ,     ,  .
/// \param [in]   strSymbol   
/// \return       +/-Lots * 1000
//+------------------------------------------------------------------------------------------+
int CVirtualOrderManager::OpenLots(string strSymbol/*=""*/)
  {
   if(strSymbol=="") strSymbol=_Symbol;
   int nLots=0;
//         ,      
   if(PositionSelect(strSymbol))
     {
      switch(PositionGetInteger(POSITION_TYPE))
        {
         case POSITION_TYPE_BUY:
            nLots=(int)MathRound(PositionGetDouble(POSITION_VOLUME)*1000);
            break;

         case POSITION_TYPE_SELL:
            nLots=-(int)MathRound(PositionGetDouble(POSITION_VOLUME)*1000);
            break;

         default:
            LogFile.Log(LOG_PRINT,__FUNCTION__," problem with POSITION_TYPE");
        }
     }
   LogFile.Log(LOG_VERBOSE,__FUNCTION__,"(",strSymbol,") returning ",(string)nLots);
   return(nLots);
  }
//+------------------------------------------------------------------+
///      .
///   Buy  Sell      .
///  StopLoss    StopLoss    + ServerStopLossMargin,   CConfig
/// \param [in]   strSymbol   
/// \param [in]   dblLots      
/// \param [in]	OrderType	     (Buy  Sell)
/// \param [in]	dblStopLoss   
/// \return                   True  ,  false 
//+------------------------------------------------------------------+
bool CVirtualOrderManager::PositionChangeSizeAtServer(string strSymbol,double dblLots,ENUM_ORDER_TYPE OrderType,double dblStopLoss)
  {
   long n=SymbolInfoInteger(strSymbol,SYMBOL_DIGITS);

   LogFile.Log(LOG_PRINT,__FUNCTION__,StringFormat("(%s,%0.2f,%s,%0."+(string)n+"f)",strSymbol,dblLots,OrderTypeToString(OrderType),dblStopLoss));

   double dblTightestLongStop=0.0;
   double dblTightestShortStop=0.0;
   m_GlobalVirtualStopList.TightestStops(strSymbol,dblTightestShortStop,dblTightestLongStop);

   MqlTick CurrentTick;
   SymbolInfoTick(strSymbol,CurrentTick);
   long lDigits=SymbolInfoInteger(strSymbol,SYMBOL_DIGITS);
   double dblPoint=SymbolInfoDouble(strSymbol,SYMBOL_POINT);

   MqlTradeRequest MtRequest;
   int nCurrentLots=OpenLots(strSymbol);
   int nTargetLots=0;
   switch(OrderType)
     {
      case ORDER_TYPE_BUY:
         nTargetLots=nCurrentLots+(int)MathRound(dblLots*1000);
         MtRequest.price=CurrentTick.ask;
         dblTightestLongStop=MathMax(dblTightestLongStop,dblStopLoss);
         MtRequest.sl=dblTightestLongStop==0.0 ? 0.0 : NormalizeDouble(dblTightestLongStop-Config.ServerStopLossMargin*dblPoint,(int)lDigits);
         break;

      case ORDER_TYPE_SELL:
         nTargetLots=nCurrentLots-(int)MathRound(dblLots*1000);
         MtRequest.price=CurrentTick.bid;
         if(dblStopLoss!=0.0) dblTightestShortStop=MathMin(dblTightestShortStop,dblStopLoss);
         if(dblTightestShortStop==DBL_MAX || dblTightestShortStop==0)
            MtRequest.sl=0.0;
         else
            MtRequest.sl=NormalizeDouble(dblTightestShortStop+Config.ServerStopLossMargin*dblPoint,(int)lDigits);
         break;

      default:
         LogFile.Log(LOG_PRINT,__FUNCTION__," error: Invalid ordertype");
         return(false);
     }

   MtRequest.tp=0.0;
   MtRequest.action= TRADE_ACTION_DEAL;
   MtRequest.magic = MagicNumber();
   MtRequest.symbol = strSymbol;
   MtRequest.volume = NormalizeDouble(dblLots,2);
   MtRequest.deviation=50;
   MtRequest.type=OrderType;
   MtRequest.type_filling=ORDER_FILLING_FOK;

   LogFile.Log(LOG_DEBUG,__FUNCTION__,StringFormat(" attempting %s for %s at %0.5G with stoploss %0.5G",OrderTypeToString(OrderType),strSymbol,MtRequest.price,MtRequest.sl));
   MqlTradeResult MtResult;
   if(!OrderSend(MtRequest,MtResult))
     {
      LogFile.Log(LOG_PRINT,__FUNCTION__," error: Problem with OrderSend TRADE_ACTION_DEAL, return code ",ReturnCodeDescription(MtResult.retcode));
      return(false);
     }
   else
     {
      LogFile.Log(LOG_DEBUG,__FUNCTION__," result of OrderSend TRADE_ACTION_DEAL to server: ",ReturnCodeDescription(MtResult.retcode));
      return(true);
     }
  }
//+------------------------------------------------------------------+
///    StopLoss.
/// ,  Virtual Order Manager   takeprofit  ,       \n
///  StopLoss    StopLoss    +  ServerStopLossMargin,    CConfig
/// \param [in]   nTicket		 ,  
/// \return       True  ,  false 
//+------------------------------------------------------------------+
bool CVirtualOrderManager::PositionSetTightestStopAtServer(string strSymbol)
  {
   long lDigits=SymbolInfoInteger(strSymbol,SYMBOL_DIGITS);
   LogFile.Log(LOG_PRINT,__FUNCTION__,StringFormat("(%d,%0."+(string)lDigits+"f)",strSymbol));

   int nPositionSize=OpenLots(strSymbol);

   if(nPositionSize==0)
     {
      LogFile.Log(LOG_PRINT,__FUNCTION__," no open position - no action");
      return(true);
     }

   double dblTightestLongStop=0.0;
   double dblTightestShortStop=0.0;
   m_GlobalVirtualStopList.TightestStops(strSymbol,dblTightestShortStop,dblTightestLongStop);

   MqlTick CurrentTick;
   SymbolInfoTick(strSymbol,CurrentTick);
   double dblPoint=SymbolInfoDouble(strSymbol,SYMBOL_POINT);
   PositionSelect(strSymbol);
   double dblPositionStopLoss=PositionGetDouble(POSITION_SL);

   MqlTradeRequest MtRequest;

   if(nPositionSize>0)
     {
      //   Stop Loss  
      MtRequest.sl=dblTightestLongStop==0.0 ? 0.0 : NormalizeDouble(dblTightestLongStop-Config.ServerStopLossMargin*dblPoint,(int)lDigits);

     }
   else if(nPositionSize<0)
     {
      //   Stop Loss  
      if(dblTightestShortStop==DBL_MAX || dblTightestShortStop==0)
         MtRequest.sl=0.0;
      else
         MtRequest.sl=NormalizeDouble(dblTightestShortStop+Config.ServerStopLossMargin*dblPoint,(int)lDigits);
     }

   if(MathRound((MtRequest.sl-dblPositionStopLoss)/dblPoint)==0)
     {
      LogFile.Log(LOG_PRINT,__FUNCTION__," POSITION_SL is already correct - no action");
      return(true);
     }

   MtRequest.action=TRADE_ACTION_SLTP;
   MtRequest.tp=0.0;
   MtRequest.symbol=strSymbol;

   MqlTradeResult MtResult;
   if(!OrderSend(MtRequest,MtResult))
     {
      LogFile.Log(LOG_PRINT,__FUNCTION__," error: Problem with OrderSend TRADE_ACTION_SLTP, return code ",ReturnCodeDescription(MtResult.retcode));
      return(false);
     }
   LogFile.Log(LOG_DEBUG,__FUNCTION__," result of OrderSend TRADE_ACTION_SLTP to server: ",ReturnCodeDescription(MtResult.retcode));
   return(true);
  }
//+------------------------------------------------------------------+
///    Sell.
/// \param strSymbol		 
/// \param nStopLoss		 Stop Loss     
/// \param nTakeProfit	 Take Profit     
/// \param strComment	   
/// \return 				     ,  -1   .
//+------------------------------------------------------------------+
long CVirtualOrderManager::Sell(string strSymbol,double dblLots,const int nStoploss,const int nTakeprofit,string strComment/*=""*/)
  {
   if(strComment=="") strComment=MQL5InfoString(MQL5_PROGRAM_NAME);
   LogFile.Log(LOG_DEBUG,__FUNCTION__,StringFormat("(%s,%0.2f,%d,%d,%s)",strSymbol,dblLots,nStoploss,nTakeprofit,strComment));
   double sl,tp;
   double dblPrice=SymbolInfoDouble(strSymbol,SYMBOL_BID);
   double dblPoint=SymbolInfoDouble(strSymbol,SYMBOL_POINT);
   sl =nStoploss==0 ? 0.0 : dblPrice+nStoploss*dblPoint;
   tp =nTakeprofit==0 ? 0.0 : dblPrice-nTakeprofit*dblPoint;
   return(VOM.OrderSend(strSymbol,VIRTUAL_ORDER_TYPE_SELL,dblLots,dblPrice,Config.Deviation,sl,tp,strComment,0));
  }
//+------------------------------------------------------------------+
